# Copyright DB InfraGO AG and contributors
# SPDX-License-Identifier: Apache-2.0

from collections.abc import Mapping
from typing import (
    TYPE_CHECKING,
    Any,
    TypeVar,
    Union,
)

from attrs import define as _attrs_define
from attrs import field as _attrs_field

from ..types import UNSET, Unset

if TYPE_CHECKING:
    from ..models.page_attachments_single_get_response_data_relationships_project_data import (
        PageAttachmentsSingleGetResponseDataRelationshipsProjectData,
    )


T = TypeVar(
    "T", bound="PageAttachmentsSingleGetResponseDataRelationshipsProject"
)


@_attrs_define
class PageAttachmentsSingleGetResponseDataRelationshipsProject:
    """
    Attributes:
        data (Union[Unset, PageAttachmentsSingleGetResponseDataRelationshipsProjectData]):
    """

    data: Union[
        Unset, "PageAttachmentsSingleGetResponseDataRelationshipsProjectData"
    ] = UNSET
    additional_properties: dict[str, Any] = _attrs_field(
        init=False, factory=dict
    )

    def to_dict(self) -> dict[str, Any]:
        data: Union[Unset, dict[str, Any]] = UNSET
        if not isinstance(self.data, Unset):
            data = self.data.to_dict()

        field_dict: dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update({})
        if data is not UNSET:
            field_dict["data"] = data

        return field_dict

    @classmethod
    def from_dict(cls: type[T], src_dict: Mapping[str, Any]) -> T:
        from ..models.page_attachments_single_get_response_data_relationships_project_data import (
            PageAttachmentsSingleGetResponseDataRelationshipsProjectData,
        )

        d = dict(src_dict)
        _data = d.pop("data", UNSET)
        data: Union[
            Unset, PageAttachmentsSingleGetResponseDataRelationshipsProjectData
        ]
        if isinstance(_data, Unset):
            data = UNSET
        else:
            data = PageAttachmentsSingleGetResponseDataRelationshipsProjectData.from_dict(
                _data
            )

        page_attachments_single_get_response_data_relationships_project_obj = (
            cls(
                data=data,
            )
        )

        page_attachments_single_get_response_data_relationships_project_obj.additional_properties = d
        return (
            page_attachments_single_get_response_data_relationships_project_obj
        )

    @property
    def additional_keys(self) -> list[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



# Copyright DB InfraGO AG and contributors
# SPDX-License-Identifier: Apache-2.0

from collections.abc import Mapping
from typing import (
    TYPE_CHECKING,
    Any,
    TypeVar,
    Union,
)

from attrs import define as _attrs_define
from attrs import field as _attrs_field

from ..types import UNSET, Unset

if TYPE_CHECKING:
    from ..models.testruns_single_get_response_data_relationships_author import (
        TestrunsSingleGetResponseDataRelationshipsAuthor,
    )
    from ..models.testruns_single_get_response_data_relationships_document import (
        TestrunsSingleGetResponseDataRelationshipsDocument,
    )
    from ..models.testruns_single_get_response_data_relationships_project import (
        TestrunsSingleGetResponseDataRelationshipsProject,
    )
    from ..models.testruns_single_get_response_data_relationships_project_span import (
        TestrunsSingleGetResponseDataRelationshipsProjectSpan,
    )
    from ..models.testruns_single_get_response_data_relationships_summary_defect import (
        TestrunsSingleGetResponseDataRelationshipsSummaryDefect,
    )
    from ..models.testruns_single_get_response_data_relationships_template import (
        TestrunsSingleGetResponseDataRelationshipsTemplate,
    )


T = TypeVar("T", bound="TestrunsSingleGetResponseDataRelationships")


@_attrs_define
class TestrunsSingleGetResponseDataRelationships:
    """
    Attributes:
        author (Union[Unset, TestrunsSingleGetResponseDataRelationshipsAuthor]):
        document (Union[Unset, TestrunsSingleGetResponseDataRelationshipsDocument]):
        project (Union[Unset, TestrunsSingleGetResponseDataRelationshipsProject]):
        project_span (Union[Unset, TestrunsSingleGetResponseDataRelationshipsProjectSpan]):
        summary_defect (Union[Unset, TestrunsSingleGetResponseDataRelationshipsSummaryDefect]):
        template (Union[Unset, TestrunsSingleGetResponseDataRelationshipsTemplate]):
    """

    author: Union[
        Unset, "TestrunsSingleGetResponseDataRelationshipsAuthor"
    ] = UNSET
    document: Union[
        Unset, "TestrunsSingleGetResponseDataRelationshipsDocument"
    ] = UNSET
    project: Union[
        Unset, "TestrunsSingleGetResponseDataRelationshipsProject"
    ] = UNSET
    project_span: Union[
        Unset, "TestrunsSingleGetResponseDataRelationshipsProjectSpan"
    ] = UNSET
    summary_defect: Union[
        Unset, "TestrunsSingleGetResponseDataRelationshipsSummaryDefect"
    ] = UNSET
    template: Union[
        Unset, "TestrunsSingleGetResponseDataRelationshipsTemplate"
    ] = UNSET
    additional_properties: dict[str, Any] = _attrs_field(
        init=False, factory=dict
    )

    def to_dict(self) -> dict[str, Any]:
        author: Union[Unset, dict[str, Any]] = UNSET
        if not isinstance(self.author, Unset):
            author = self.author.to_dict()

        document: Union[Unset, dict[str, Any]] = UNSET
        if not isinstance(self.document, Unset):
            document = self.document.to_dict()

        project: Union[Unset, dict[str, Any]] = UNSET
        if not isinstance(self.project, Unset):
            project = self.project.to_dict()

        project_span: Union[Unset, dict[str, Any]] = UNSET
        if not isinstance(self.project_span, Unset):
            project_span = self.project_span.to_dict()

        summary_defect: Union[Unset, dict[str, Any]] = UNSET
        if not isinstance(self.summary_defect, Unset):
            summary_defect = self.summary_defect.to_dict()

        template: Union[Unset, dict[str, Any]] = UNSET
        if not isinstance(self.template, Unset):
            template = self.template.to_dict()

        field_dict: dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update({})
        if author is not UNSET:
            field_dict["author"] = author
        if document is not UNSET:
            field_dict["document"] = document
        if project is not UNSET:
            field_dict["project"] = project
        if project_span is not UNSET:
            field_dict["projectSpan"] = project_span
        if summary_defect is not UNSET:
            field_dict["summaryDefect"] = summary_defect
        if template is not UNSET:
            field_dict["template"] = template

        return field_dict

    @classmethod
    def from_dict(cls: type[T], src_dict: Mapping[str, Any]) -> T:
        from ..models.testruns_single_get_response_data_relationships_author import (
            TestrunsSingleGetResponseDataRelationshipsAuthor,
        )
        from ..models.testruns_single_get_response_data_relationships_document import (
            TestrunsSingleGetResponseDataRelationshipsDocument,
        )
        from ..models.testruns_single_get_response_data_relationships_project import (
            TestrunsSingleGetResponseDataRelationshipsProject,
        )
        from ..models.testruns_single_get_response_data_relationships_project_span import (
            TestrunsSingleGetResponseDataRelationshipsProjectSpan,
        )
        from ..models.testruns_single_get_response_data_relationships_summary_defect import (
            TestrunsSingleGetResponseDataRelationshipsSummaryDefect,
        )
        from ..models.testruns_single_get_response_data_relationships_template import (
            TestrunsSingleGetResponseDataRelationshipsTemplate,
        )

        d = dict(src_dict)
        _author = d.pop("author", UNSET)
        author: Union[Unset, TestrunsSingleGetResponseDataRelationshipsAuthor]
        if isinstance(_author, Unset):
            author = UNSET
        else:
            author = (
                TestrunsSingleGetResponseDataRelationshipsAuthor.from_dict(
                    _author
                )
            )

        _document = d.pop("document", UNSET)
        document: Union[
            Unset, TestrunsSingleGetResponseDataRelationshipsDocument
        ]
        if isinstance(_document, Unset):
            document = UNSET
        else:
            document = (
                TestrunsSingleGetResponseDataRelationshipsDocument.from_dict(
                    _document
                )
            )

        _project = d.pop("project", UNSET)
        project: Union[
            Unset, TestrunsSingleGetResponseDataRelationshipsProject
        ]
        if isinstance(_project, Unset):
            project = UNSET
        else:
            project = (
                TestrunsSingleGetResponseDataRelationshipsProject.from_dict(
                    _project
                )
            )

        _project_span = d.pop("projectSpan", UNSET)
        project_span: Union[
            Unset, TestrunsSingleGetResponseDataRelationshipsProjectSpan
        ]
        if isinstance(_project_span, Unset):
            project_span = UNSET
        else:
            project_span = TestrunsSingleGetResponseDataRelationshipsProjectSpan.from_dict(
                _project_span
            )

        _summary_defect = d.pop("summaryDefect", UNSET)
        summary_defect: Union[
            Unset, TestrunsSingleGetResponseDataRelationshipsSummaryDefect
        ]
        if isinstance(_summary_defect, Unset):
            summary_defect = UNSET
        else:
            summary_defect = TestrunsSingleGetResponseDataRelationshipsSummaryDefect.from_dict(
                _summary_defect
            )

        _template = d.pop("template", UNSET)
        template: Union[
            Unset, TestrunsSingleGetResponseDataRelationshipsTemplate
        ]
        if isinstance(_template, Unset):
            template = UNSET
        else:
            template = (
                TestrunsSingleGetResponseDataRelationshipsTemplate.from_dict(
                    _template
                )
            )

        testruns_single_get_response_data_relationships_obj = cls(
            author=author,
            document=document,
            project=project,
            project_span=project_span,
            summary_defect=summary_defect,
            template=template,
        )

        testruns_single_get_response_data_relationships_obj.additional_properties = d
        return testruns_single_get_response_data_relationships_obj

    @property
    def additional_keys(self) -> list[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



# Copyright DB InfraGO AG and contributors
# SPDX-License-Identifier: Apache-2.0

from collections.abc import Mapping
from typing import (
    Any,
    TypeVar,
    Union,
)

from attrs import define as _attrs_define
from attrs import field as _attrs_field

from ..models.featureselections_single_get_response_data_attributes_selection_type import (
    FeatureselectionsSingleGetResponseDataAttributesSelectionType,
)
from ..types import UNSET, Unset

T = TypeVar("T", bound="FeatureselectionsSingleGetResponseDataAttributes")


@_attrs_define
class FeatureselectionsSingleGetResponseDataAttributes:
    """
    Attributes:
        selection_type (Union[Unset, FeatureselectionsSingleGetResponseDataAttributesSelectionType]):  Example:
            excluded.
    """

    selection_type: Union[
        Unset, FeatureselectionsSingleGetResponseDataAttributesSelectionType
    ] = UNSET
    additional_properties: dict[str, Any] = _attrs_field(
        init=False, factory=dict
    )

    def to_dict(self) -> dict[str, Any]:
        selection_type: Union[Unset, str] = UNSET
        if not isinstance(self.selection_type, Unset):
            selection_type = self.selection_type.value

        field_dict: dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update({})
        if selection_type is not UNSET:
            field_dict["selectionType"] = selection_type

        return field_dict

    @classmethod
    def from_dict(cls: type[T], src_dict: Mapping[str, Any]) -> T:
        d = dict(src_dict)
        _selection_type = d.pop("selectionType", UNSET)
        selection_type: Union[
            Unset,
            FeatureselectionsSingleGetResponseDataAttributesSelectionType,
        ]
        if isinstance(_selection_type, Unset):
            selection_type = UNSET
        else:
            selection_type = (
                FeatureselectionsSingleGetResponseDataAttributesSelectionType(
                    _selection_type
                )
            )

        featureselections_single_get_response_data_attributes_obj = cls(
            selection_type=selection_type,
        )

        featureselections_single_get_response_data_attributes_obj.additional_properties = d
        return featureselections_single_get_response_data_attributes_obj

    @property
    def additional_keys(self) -> list[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



# Copyright DB InfraGO AG and contributors
# SPDX-License-Identifier: Apache-2.0

from http import HTTPStatus
from typing import Any, Union, cast

import httpx

from ... import errors
from ...client import AuthenticatedClient, Client
from ...models.errors import Errors
from ...models.teststep_results_single_patch_request import (
    TeststepResultsSinglePatchRequest,
)
from ...types import Response


def _get_kwargs(
    project_id: str,
    test_run_id: str,
    test_case_project_id: str,
    test_case_id: str,
    iteration: str,
    test_step_index: str,
    *,
    body: TeststepResultsSinglePatchRequest,
) -> dict[str, Any]:
    headers: dict[str, Any] = {}

    _kwargs: dict[str, Any] = {
        "method": "patch",
        "url": f"/projects/{project_id}/testruns/{test_run_id}/testrecords/{test_case_project_id}/{test_case_id}/{iteration}/teststepresults/{test_step_index}",
    }

    _kwargs["json"] = body.to_dict()

    headers["Content-Type"] = "application/json"

    _kwargs["headers"] = headers
    return _kwargs


def _parse_response(
    *, client: Union[AuthenticatedClient, Client], response: httpx.Response
) -> Union[Any, Errors] | None:
    if response.status_code == 204:
        response_204 = cast(Any, None)
        return response_204
    if response.status_code == 400:
        response_400 = Errors.from_dict(response.json())

        return response_400
    if response.status_code == 401:
        response_401 = Errors.from_dict(response.json())

        return response_401
    if response.status_code == 403:
        response_403 = Errors.from_dict(response.json())

        return response_403
    if response.status_code == 404:
        response_404 = Errors.from_dict(response.json())

        return response_404
    if response.status_code == 409:
        response_409 = Errors.from_dict(response.json())

        return response_409
    if response.status_code == 413:
        response_413 = Errors.from_dict(response.json())

        return response_413
    if response.status_code == 415:
        response_415 = Errors.from_dict(response.json())

        return response_415
    if response.status_code == 500:
        response_500 = Errors.from_dict(response.json())

        return response_500
    if response.status_code == 503:
        response_503 = Errors.from_dict(response.json())

        return response_503
    if client.raise_on_unexpected_status:
        raise errors.UnexpectedStatus(response.status_code, response.content)
    return None


def _build_response(
    *, client: Union[AuthenticatedClient, Client], response: httpx.Response
) -> Response[Union[Any, Errors]]:
    return Response(
        status_code=HTTPStatus(response.status_code),
        content=response.content,
        headers=response.headers,
        parsed=_parse_response(client=client, response=response),
    )


def sync_detailed(
    project_id: str,
    test_run_id: str,
    test_case_project_id: str,
    test_case_id: str,
    iteration: str,
    test_step_index: str,
    *,
    client: Union[AuthenticatedClient, Client],
    body: TeststepResultsSinglePatchRequest,
) -> Response[Union[Any, Errors]]:
    """Updates the specified Test Step Result.

    Args:
        project_id (str):
        test_run_id (str):
        test_case_project_id (str):
        test_case_id (str):
        iteration (str):
        test_step_index (str):
        body (TeststepResultsSinglePatchRequest):

    Raises:
        errors.UnexpectedStatus: If the server returns an undocumented status code and Client.raise_on_unexpected_status is True.
        httpx.TimeoutException: If the request takes longer than Client.timeout.

    Returns:
        Response[Union[Any, Errors]]
    """

    kwargs = _get_kwargs(
        project_id=project_id,
        test_run_id=test_run_id,
        test_case_project_id=test_case_project_id,
        test_case_id=test_case_id,
        iteration=iteration,
        test_step_index=test_step_index,
        body=body,
    )

    response = client.get_httpx_client().request(
        **kwargs,
    )

    return _build_response(client=client, response=response)


def sync(
    project_id: str,
    test_run_id: str,
    test_case_project_id: str,
    test_case_id: str,
    iteration: str,
    test_step_index: str,
    *,
    client: Union[AuthenticatedClient, Client],
    body: TeststepResultsSinglePatchRequest,
) -> Union[Any, Errors] | None:
    """Updates the specified Test Step Result.

    Args:
        project_id (str):
        test_run_id (str):
        test_case_project_id (str):
        test_case_id (str):
        iteration (str):
        test_step_index (str):
        body (TeststepResultsSinglePatchRequest):

    Raises:
        errors.UnexpectedStatus: If the server returns an undocumented status code and Client.raise_on_unexpected_status is True.
        httpx.TimeoutException: If the request takes longer than Client.timeout.

    Returns:
        Union[Any, Errors]
    """

    return sync_detailed(
        project_id=project_id,
        test_run_id=test_run_id,
        test_case_project_id=test_case_project_id,
        test_case_id=test_case_id,
        iteration=iteration,
        test_step_index=test_step_index,
        client=client,
        body=body,
    ).parsed


async def asyncio_detailed(
    project_id: str,
    test_run_id: str,
    test_case_project_id: str,
    test_case_id: str,
    iteration: str,
    test_step_index: str,
    *,
    client: Union[AuthenticatedClient, Client],
    body: TeststepResultsSinglePatchRequest,
) -> Response[Union[Any, Errors]]:
    """Updates the specified Test Step Result.

    Args:
        project_id (str):
        test_run_id (str):
        test_case_project_id (str):
        test_case_id (str):
        iteration (str):
        test_step_index (str):
        body (TeststepResultsSinglePatchRequest):

    Raises:
        errors.UnexpectedStatus: If the server returns an undocumented status code and Client.raise_on_unexpected_status is True.
        httpx.TimeoutException: If the request takes longer than Client.timeout.

    Returns:
        Response[Union[Any, Errors]]
    """

    kwargs = _get_kwargs(
        project_id=project_id,
        test_run_id=test_run_id,
        test_case_project_id=test_case_project_id,
        test_case_id=test_case_id,
        iteration=iteration,
        test_step_index=test_step_index,
        body=body,
    )

    response = await client.get_async_httpx_client().request(**kwargs)

    return _build_response(client=client, response=response)


async def asyncio(
    project_id: str,
    test_run_id: str,
    test_case_project_id: str,
    test_case_id: str,
    iteration: str,
    test_step_index: str,
    *,
    client: Union[AuthenticatedClient, Client],
    body: TeststepResultsSinglePatchRequest,
) -> Union[Any, Errors] | None:
    """Updates the specified Test Step Result.

    Args:
        project_id (str):
        test_run_id (str):
        test_case_project_id (str):
        test_case_id (str):
        iteration (str):
        test_step_index (str):
        body (TeststepResultsSinglePatchRequest):

    Raises:
        errors.UnexpectedStatus: If the server returns an undocumented status code and Client.raise_on_unexpected_status is True.
        httpx.TimeoutException: If the request takes longer than Client.timeout.

    Returns:
        Union[Any, Errors]
    """

    return (
        await asyncio_detailed(
            project_id=project_id,
            test_run_id=test_run_id,
            test_case_project_id=test_case_project_id,
            test_case_id=test_case_id,
            iteration=iteration,
            test_step_index=test_step_index,
            client=client,
            body=body,
        )
    ).parsed



# Copyright DB InfraGO AG and contributors
# SPDX-License-Identifier: Apache-2.0

from enum import Enum


class DocumentsSinglePostResponseDataRelationshipsCommentsDataItemType(
    str, Enum
):
    DOCUMENT_COMMENTS = "document_comments"

    def __str__(self) -> str:
        return str(self.value)



# Copyright DB InfraGO AG and contributors
# SPDX-License-Identifier: Apache-2.0

from collections.abc import Mapping
from typing import (
    TYPE_CHECKING,
    Any,
    TypeVar,
    Union,
)

from attrs import define as _attrs_define
from attrs import field as _attrs_field

from ..types import UNSET, Unset

if TYPE_CHECKING:
    from ..models.workitem_approvals_list_get_response_data_item import (
        WorkitemApprovalsListGetResponseDataItem,
    )
    from ..models.workitem_approvals_list_get_response_included_item import (
        WorkitemApprovalsListGetResponseIncludedItem,
    )
    from ..models.workitem_approvals_list_get_response_links import (
        WorkitemApprovalsListGetResponseLinks,
    )
    from ..models.workitem_approvals_list_get_response_meta import (
        WorkitemApprovalsListGetResponseMeta,
    )


T = TypeVar("T", bound="WorkitemApprovalsListGetResponse")


@_attrs_define
class WorkitemApprovalsListGetResponse:
    """
    Attributes:
        data (Union[Unset, list['WorkitemApprovalsListGetResponseDataItem']]):
        included (Union[Unset, list['WorkitemApprovalsListGetResponseIncludedItem']]): Related entities might be
            returned, see <a href="https://docs.sw.siemens.com/en-
            US/doc/230235217/PL20241023686685479.polarion_help_sc.xid2134849/xid2134871" target="_blank">REST API User
            Guide</a>.
        links (Union[Unset, WorkitemApprovalsListGetResponseLinks]):
        meta (Union[Unset, WorkitemApprovalsListGetResponseMeta]):
    """

    data: Union[Unset, list["WorkitemApprovalsListGetResponseDataItem"]] = (
        UNSET
    )
    included: Union[
        Unset, list["WorkitemApprovalsListGetResponseIncludedItem"]
    ] = UNSET
    links: Union[Unset, "WorkitemApprovalsListGetResponseLinks"] = UNSET
    meta: Union[Unset, "WorkitemApprovalsListGetResponseMeta"] = UNSET
    additional_properties: dict[str, Any] = _attrs_field(
        init=False, factory=dict
    )

    def to_dict(self) -> dict[str, Any]:
        data: Union[Unset, list[dict[str, Any]]] = UNSET
        if not isinstance(self.data, Unset):
            data = []
            for data_item_data in self.data:
                data_item = data_item_data.to_dict()
                data.append(data_item)

        included: Union[Unset, list[dict[str, Any]]] = UNSET
        if not isinstance(self.included, Unset):
            included = []
            for included_item_data in self.included:
                included_item = included_item_data.to_dict()
                included.append(included_item)

        links: Union[Unset, dict[str, Any]] = UNSET
        if not isinstance(self.links, Unset):
            links = self.links.to_dict()

        meta: Union[Unset, dict[str, Any]] = UNSET
        if not isinstance(self.meta, Unset):
            meta = self.meta.to_dict()

        field_dict: dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update({})
        if data is not UNSET:
            field_dict["data"] = data
        if included is not UNSET:
            field_dict["included"] = included
        if links is not UNSET:
            field_dict["links"] = links
        if meta is not UNSET:
            field_dict["meta"] = meta

        return field_dict

    @classmethod
    def from_dict(cls: type[T], src_dict: Mapping[str, Any]) -> T:
        from ..models.workitem_approvals_list_get_response_data_item import (
            WorkitemApprovalsListGetResponseDataItem,
        )
        from ..models.workitem_approvals_list_get_response_included_item import (
            WorkitemApprovalsListGetResponseIncludedItem,
        )
        from ..models.workitem_approvals_list_get_response_links import (
            WorkitemApprovalsListGetResponseLinks,
        )
        from ..models.workitem_approvals_list_get_response_meta import (
            WorkitemApprovalsListGetResponseMeta,
        )

        d = dict(src_dict)
        data = []
        _data = d.pop("data", UNSET)
        for data_item_data in _data or []:
            data_item = WorkitemApprovalsListGetResponseDataItem.from_dict(
                data_item_data
            )

            data.append(data_item)

        included = []
        _included = d.pop("included", UNSET)
        for included_item_data in _included or []:
            included_item = (
                WorkitemApprovalsListGetResponseIncludedItem.from_dict(
                    included_item_data
                )
            )

            included.append(included_item)

        _links = d.pop("links", UNSET)
        links: Union[Unset, WorkitemApprovalsListGetResponseLinks]
        if isinstance(_links, Unset):
            links = UNSET
        else:
            links = WorkitemApprovalsListGetResponseLinks.from_dict(_links)

        _meta = d.pop("meta", UNSET)
        meta: Union[Unset, WorkitemApprovalsListGetResponseMeta]
        if isinstance(_meta, Unset):
            meta = UNSET
        else:
            meta = WorkitemApprovalsListGetResponseMeta.from_dict(_meta)

        workitem_approvals_list_get_response_obj = cls(
            data=data,
            included=included,
            links=links,
            meta=meta,
        )

        workitem_approvals_list_get_response_obj.additional_properties = d
        return workitem_approvals_list_get_response_obj

    @property
    def additional_keys(self) -> list[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



# Copyright DB InfraGO AG and contributors
# SPDX-License-Identifier: Apache-2.0
"""Implementation of LinkedWorkItems operations."""

import itertools
import typing as t

from polarion_rest_api_client import data_models as dm
from polarion_rest_api_client.open_api_client import models as api_models
from polarion_rest_api_client.open_api_client import types as oa_types
from polarion_rest_api_client.open_api_client.api.linked_work_items import (
    delete_linked_work_items,
    get_linked_work_items,
    post_linked_work_items,
)

from . import base_classes as bc

LINK_ID_PART_COUNT = 5


class WorkItemLinks(
    bc.CreateClient[dm.WorkItemLink],
    bc.MultiGetClient[dm.WorkItemLink],
    bc.DeleteClient[dm.WorkItemLink],
):
    """A client providing LinkedWorkItems functions."""

    def get_multi(  # type: ignore[override]
        self,
        work_item_id: str,
        *,
        page_size: int = 100,
        page_number: int = 1,
        fields: dict[str, str] | None = None,
        include: str | None | oa_types.Unset = None,
        revision: str | None | oa_types.Unset = None,
    ) -> tuple[list[dm.WorkItemLink], bool]:
        """Get the work item links for the given work item on a page.

        In addition, a flag whether a next page is available is
        returned. Define a fields dictionary as described in the
        Polarion API documentation to get certain fields.
        """
        if fields is None:
            fields = self._client.default_fields.linkedworkitems

        response = get_linked_work_items.sync_detailed(
            self._project_id,
            work_item_id,
            client=self._client.client,
            fields=self._build_sparse_fields(fields),
            include=self.none_to_unset(include),
            pagesize=page_size,
            pagenumber=page_number,
            revision=revision or oa_types.UNSET,
        )

        return self._parse_get_response(response, work_item_id)

    async def async_get_multi(  # type: ignore[override]
        self,
        work_item_id: str,
        *,
        page_size: int = 100,
        page_number: int = 1,
        fields: dict[str, str] | None = None,
        include: str | None | oa_types.Unset = None,
        revision: str | None | oa_types.Unset = None,
    ) -> tuple[list[dm.WorkItemLink], bool]:
        """Get the work item links for the given work item on a page.

        In addition, a flag whether a next page is available is
        returned. Define a fields dictionary as described in the
        Polarion API documentation to get certain fields.
        """
        if fields is None:
            fields = self._client.default_fields.linkedworkitems

        response = await get_linked_work_items.asyncio_detailed(
            self._project_id,
            work_item_id,
            client=self._client.client,
            fields=self._build_sparse_fields(fields),
            include=self.none_to_unset(include),
            pagesize=page_size,
            pagenumber=page_number,
            revision=revision or oa_types.UNSET,
        )

        return self._parse_get_response(response, work_item_id)

    def _parse_get_response(
        self, response: oa_types.Response, work_item_id: str
    ) -> tuple[list[dm.WorkItemLink], bool]:
        self._raise_on_error(response)
        linked_work_item_response = response.parsed
        work_item_links: list[dm.WorkItemLink] = []
        next_page = False
        if (
            isinstance(
                linked_work_item_response,
                api_models.LinkedworkitemsListGetResponse,
            )
            and linked_work_item_response.data
        ):
            for link in linked_work_item_response.data:
                assert isinstance(link.id, str)
                assert isinstance(
                    link.attributes,
                    api_models.LinkedworkitemsListGetResponseDataItemAttributes,
                    # pylint: disable=line-too-long
                )

                work_item_links.append(
                    self._parse_work_item_link(
                        link.id,
                        link.attributes.suspect,
                        work_item_id,
                        link.attributes.revision,
                    )
                )

            next_page = isinstance(
                linked_work_item_response.links,
                api_models.LinkedworkitemsListGetResponseLinks,
            ) and bool(linked_work_item_response.links.next_)
        return work_item_links, next_page

    def _parse_work_item_link(
        self,
        link_id: str,
        suspect: bool | oa_types.Unset,
        work_item_id: str,
        revision: str | oa_types.Unset,
    ) -> dm.WorkItemLink:
        info = link_id.split("/")
        assert len(info) == LINK_ID_PART_COUNT
        role_id, target_project_id, linked_work_item_id = info[2:]

        return dm.WorkItemLink(
            work_item_id,
            linked_work_item_id,
            role_id,
            None if isinstance(suspect, oa_types.Unset) else suspect,
            target_project_id,
            None if isinstance(revision, oa_types.Unset) else revision,
        )

    def _pre_batching_grouping(
        self, items: list[dm.WorkItemLink]
    ) -> t.Generator[list[dm.WorkItemLink], None, None]:
        for _, group in itertools.groupby(
            items, lambda x: x.primary_work_item_id
        ):
            yield list(group)

    def _create(self, items: list[dm.WorkItemLink]) -> None:
        response = post_linked_work_items.sync_detailed(
            self._project_id,
            items[0].primary_work_item_id,
            client=self._client.client,
            body=self._create_post_body(items),
        )

        self._raise_on_error(response)

    async def _async_create(self, items: list[dm.WorkItemLink]) -> None:
        response = await post_linked_work_items.asyncio_detailed(
            self._project_id,
            items[0].primary_work_item_id,
            client=self._client.client,
            body=self._create_post_body(items),
        )

        self._raise_on_error(response)

    def _create_post_body(
        self, items: list[dm.WorkItemLink]
    ) -> api_models.LinkedworkitemsListPostRequest:
        # pylint: disable=line-too-long
        return api_models.LinkedworkitemsListPostRequest(
            data=[
                api_models.LinkedworkitemsListPostRequestDataItem(
                    type_=api_models.LinkedworkitemsListPostRequestDataItemType.LINKEDWORKITEMS,
                    attributes=api_models.LinkedworkitemsListPostRequestDataItemAttributes(
                        role=work_item_link.role,
                        suspect=work_item_link.suspect or False,
                        revision=work_item_link.secondary_work_item_revision
                        or oa_types.UNSET,
                    ),
                    relationships=api_models.LinkedworkitemsListPostRequestDataItemRelationships(
                        work_item=api_models.LinkedworkitemsListPostRequestDataItemRelationshipsWorkItem(
                            data=api_models.LinkedworkitemsListPostRequestDataItemRelationshipsWorkItemData(
                                type_=api_models.LinkedworkitemsListPostRequestDataItemRelationshipsWorkItemDataType.WORKITEMS,
                                id=f"{work_item_link.secondary_work_item_project or self._project_id}/{work_item_link.secondary_work_item_id}",
                            )
                        )
                    ),
                )
                for work_item_link in items
            ]
        )

    # pylint: enable=line-too-long

    def _delete(self, items: list[dm.WorkItemLink]) -> None:
        response = delete_linked_work_items.sync_detailed(
            self._project_id,
            items[0].primary_work_item_id,
            client=self._client.client,
            body=self._create_delete_body(items),
        )
        self._raise_on_error(response)

    async def _async_delete(self, items: list[dm.WorkItemLink]) -> None:
        response = await delete_linked_work_items.asyncio_detailed(
            self._project_id,
            items[0].primary_work_item_id,
            client=self._client.client,
            body=self._create_delete_body(items),
        )
        self._raise_on_error(response)

    def _create_delete_body(
        self, items: list[dm.WorkItemLink]
    ) -> api_models.LinkedworkitemsListDeleteRequest:
        # pylint: disable=line-too-long
        return api_models.LinkedworkitemsListDeleteRequest(
            data=[
                api_models.LinkedworkitemsListDeleteRequestDataItem(
                    type_=api_models.LinkedworkitemsListDeleteRequestDataItemType.LINKEDWORKITEMS,
                    id=f"{self._project_id}/{work_item_link.primary_work_item_id}/{work_item_link.role}/{work_item_link.secondary_work_item_project or self._project_id}/{work_item_link.secondary_work_item_id}",
                )
                for work_item_link in items
            ]
        )
        # pylint: enable=line-too-long



# Copyright DB InfraGO AG and contributors
# SPDX-License-Identifier: Apache-2.0
"""Clients to handle test parameters."""

import itertools
import typing as t

from polarion_rest_api_client import data_models as dm
from polarion_rest_api_client.open_api_client import models as api_models
from polarion_rest_api_client.open_api_client.api.test_records import (
    delete_test_record_test_parameter,
    get_test_record_test_parameters,
    post_test_record_test_parameters,
)
from polarion_rest_api_client.open_api_client.api.test_runs import (
    delete_test_run_test_parameters,
    get_test_run_test_parameters,
    post_test_run_test_parameters,
)

from . import base_classes as bc


class TestRunParameters(
    bc.MultiGetClient[dm.TestRunParameter],
    bc.CreateClient[dm.TestRunParameter],
    bc.DeleteClient[dm.TestRunParameter],
):
    """Client to handle TestParameters of a TestRun."""

    def get_multi(  # type: ignore[override]
        self,
        test_run_id: str,
        *,
        page_size: int = 100,
        page_number: int = 1,
        fields: dict[str, str] | None = None,
    ) -> tuple[list[dm.TestRunParameter], bool]:
        if fields is None:
            fields = self._client.default_fields.testparameters

        sparse_fields = self._build_sparse_fields(fields)
        response = get_test_run_test_parameters.sync_detailed(
            self._project_id,
            test_run_id,
            client=self._client.client,
            fields=sparse_fields,
            pagenumber=page_number,
            pagesize=page_size,
        )

        return _parse_get_response(response.parsed, test_run_id)

    async def async_get_multi(  # type: ignore[override]
        self,
        test_run_id: str,
        *,
        page_size: int = 100,
        page_number: int = 1,
        fields: dict[str, str] | None = None,
    ) -> tuple[list[dm.TestRunParameter], bool]:
        if fields is None:
            fields = self._client.default_fields.testparameters

        sparse_fields = self._build_sparse_fields(fields)
        response = await get_test_run_test_parameters.asyncio_detailed(
            self._project_id,
            test_run_id,
            client=self._client.client,
            fields=sparse_fields,
            pagenumber=page_number,
            pagesize=page_size,
        )

        self._raise_on_error(response)
        return _parse_get_response(response.parsed, test_run_id)

    def _pre_batching_grouping(
        self, items: list[dm.TestRunParameter]
    ) -> t.Generator[list[dm.TestRunParameter], None, None]:
        for _, group in itertools.groupby(
            sorted(
                items,
                key=lambda x: x.test_run_id,
            ),
            lambda x: x.test_run_id,
        ):
            yield list(group)

    def _create(self, items: list[dm.TestRunParameter]) -> None:
        """Call only with items with common test_run_id."""
        test_run_id = items[0].test_run_id
        body = _build_post_body(items)

        response = post_test_run_test_parameters.sync_detailed(
            self._project_id,
            test_run_id,
            client=self._client.client,
            body=body,
        )
        self._raise_on_error(response)

        assert isinstance(
            response.parsed, api_models.TestparametersListPostResponse
        )
        assert response.parsed.data

    async def _async_create(self, items: list[dm.TestRunParameter]) -> None:
        """Call only with items with common test_run_id."""
        test_run_id = items[0].test_run_id
        body = _build_post_body(items)

        response = await post_test_run_test_parameters.asyncio_detailed(
            self._project_id,
            test_run_id,
            client=self._client.client,
            body=body,
        )
        self._raise_on_error(response)

        assert isinstance(
            response.parsed, api_models.TestparametersListPostResponse
        )
        assert response.parsed.data

    def _delete(self, items: list[dm.TestRunParameter]) -> None:
        """Call only with items with common test_run_id."""
        test_run_id = items[0].test_run_id
        body = self._build_delete_body(items)

        response = delete_test_run_test_parameters.sync_detailed(
            self._project_id,
            test_run_id,
            client=self._client.client,
            body=body,
        )
        self._raise_on_error(response)

    async def _async_delete(self, items: list[dm.TestRunParameter]) -> None:
        """Call only with items with common test_run_id."""
        test_run_id = items[0].test_run_id
        body = self._build_delete_body(items)

        response = await delete_test_run_test_parameters.asyncio_detailed(
            self._project_id,
            test_run_id,
            client=self._client.client,
            body=body,
        )
        self._raise_on_error(response)

    def _build_delete_body(
        self, items: list[dm.TestRunParameter]
    ) -> api_models.TestparametersListDeleteRequest:
        return api_models.TestparametersListDeleteRequest(
            data=[
                api_models.TestparametersListDeleteRequestDataItem(
                    type_=api_models.TestparametersListDeleteRequestDataItemType.TESTPARAMETERS,
                    id=f"{self._project_id}/{item.test_run_id}/{item.name}",
                )
                for item in items
            ]
        )


class TestRecordParameters(
    bc.MultiGetClient[dm.TestRecordParameter],
    bc.CreateClient[dm.TestRecordParameter],
    bc.DeleteClient[dm.TestRecordParameter],
):
    """Clients to handle TestParameters of a TestRecord."""

    _delete_batch_size = 1

    def get_multi(  # type: ignore[override]
        self,
        test_record: dm.TestRecord,
        *,
        page_size: int = 100,
        page_number: int = 1,
        fields: dict[str, str] | None = None,
    ) -> tuple[list[dm.TestRecordParameter], bool]:
        if fields is None:
            fields = self._client.default_fields.testparameters

        sparse_fields = self._build_sparse_fields(fields)
        response = get_test_record_test_parameters.sync_detailed(
            self._project_id,
            test_record.test_run_id,
            test_record.work_item_project_id,
            test_record.work_item_id,
            str(test_record.iteration),
            client=self._client.client,
            fields=sparse_fields,
            pagenumber=page_number,
            pagesize=page_size,
        )

        self._raise_on_error(response)

        return _parse_get_response(response.parsed, test_record)

    async def async_get_multi(  # type: ignore[override]
        self,
        test_record: dm.TestRecord,
        *,
        page_size: int = 100,
        page_number: int = 1,
        fields: dict[str, str] | None = None,
    ) -> tuple[list[dm.TestRecordParameter], bool]:
        if fields is None:
            fields = self._client.default_fields.testparameters

        sparse_fields = self._build_sparse_fields(fields)
        response = await get_test_record_test_parameters.asyncio_detailed(
            self._project_id,
            test_record.test_run_id,
            test_record.work_item_project_id,
            test_record.work_item_id,
            str(test_record.iteration),
            client=self._client.client,
            fields=sparse_fields,
            pagenumber=page_number,
            pagesize=page_size,
        )

        self._raise_on_error(response)

        return _parse_get_response(response.parsed, test_record)

    def _pre_batching_grouping(
        self, items: list[dm.TestRecordParameter]
    ) -> t.Generator[list[dm.TestRecordParameter], None, None]:
        for _, group in itertools.groupby(
            sorted(
                items,
                key=lambda x: (
                    f"{x.test_record.test_run_id}/{x.test_record.work_item_project_id}/{x.test_record.work_item_id}/{x.test_record.iteration}"
                ),
            ),
            lambda x: x.test_record,
        ):
            yield list(group)

    def _create(self, items: list[dm.TestRecordParameter]) -> None:
        """Call only with items with common test_records."""
        test_record = items[0].test_record
        body = _build_post_body(items)

        response = post_test_record_test_parameters.sync_detailed(
            self._project_id,
            test_record.test_run_id,
            test_record.work_item_project_id,
            test_record.work_item_id,
            str(test_record.iteration),
            client=self._client.client,
            body=body,
        )
        self._raise_on_error(response)

    async def _async_create(self, items: list[dm.TestRecordParameter]) -> None:
        """Call only with items with common test_records."""
        test_record = items[0].test_record
        body = _build_post_body(items)

        response = await post_test_record_test_parameters.asyncio_detailed(
            self._project_id,
            test_record.test_run_id,
            test_record.work_item_project_id,
            test_record.work_item_id,
            str(test_record.iteration),
            client=self._client.client,
            body=body,
        )
        self._raise_on_error(response)

    def _delete(self, items: list[dm.TestRecordParameter]) -> None:
        """We expect only one TestRecordParameter for deletion."""
        assert len(items) == 1
        test_record = items[0].test_record

        response = delete_test_record_test_parameter.sync_detailed(
            self._project_id,
            test_record.test_run_id,
            test_record.work_item_project_id,
            test_record.work_item_id,
            str(test_record.iteration),
            items[0].name,
            client=self._client.client,
        )
        self._raise_on_error(response)

    async def _async_delete(self, items: list[dm.TestRecordParameter]) -> None:
        """We expect only one TestRecordParameter for deletion."""
        assert len(items) == 1
        test_record = items[0].test_record

        response = await delete_test_record_test_parameter.asyncio_detailed(
            self._project_id,
            test_record.test_run_id,
            test_record.work_item_project_id,
            test_record.work_item_id,
            str(test_record.iteration),
            items[0].name,
            client=self._client.client,
        )
        self._raise_on_error(response)


@t.overload
def _parse_get_response(
    parsed_response: (
        api_models.TestparametersListGetResponse | api_models.Errors | None
    ),
    scope: dm.TestRecord,
) -> tuple[list[dm.TestRecordParameter], bool]: ...


@t.overload
def _parse_get_response(
    parsed_response: (
        api_models.TestparametersListGetResponse | api_models.Errors | None
    ),
    scope: str,
) -> tuple[list[dm.TestRunParameter], bool]: ...


def _parse_get_response(
    parsed_response: (
        api_models.TestparametersListGetResponse | api_models.Errors | None
    ),
    scope: str | dm.TestRecord,
) -> tuple[list[dm.TestRecordParameter] | list[dm.TestRunParameter], bool]:
    assert isinstance(
        parsed_response, api_models.TestparametersListGetResponse
    )
    next_page = isinstance(
        parsed_response.links, api_models.TestparametersListGetResponseLinks
    ) and bool(parsed_response.links.next_)
    if isinstance(scope, str):
        return [
            dm.TestRunParameter(
                test_run_id=scope,
                name=data.attributes.name or "",
                value=data.attributes.value or "",
            )
            for data in parsed_response.data or []
            if data.attributes
        ], next_page

    return [
        dm.TestRecordParameter(
            test_record=scope,
            name=data.attributes.name or "",
            value=data.attributes.value or "",
        )
        for data in parsed_response.data or []
        if data.attributes
    ], next_page


def _build_post_body(
    items: list[dm.TestRunParameter] | list[dm.TestRecordParameter],
) -> api_models.TestparametersListPostRequest:
    return api_models.TestparametersListPostRequest(
        data=[
            api_models.TestparametersListPostRequestDataItem(
                type_=api_models.TestparametersListPostRequestDataItemType.TESTPARAMETERS,
                attributes=api_models.TestparametersListPostRequestDataItemAttributes(
                    name=item.name, value=item.value
                ),
            )
            for item in items
        ]
    )



# Copyright DB InfraGO AG and contributors
# SPDX-License-Identifier: Apache-2.0

from http import HTTPStatus
from typing import Any, Union, cast

import httpx

from ... import errors
from ...client import AuthenticatedClient, Client
from ...models.errors import Errors
from ...types import UNSET, Response, Unset


def _get_kwargs(
    project_id: str,
    space_id: str,
    page_name: str,
    attachment_id: str,
    *,
    revision: Union[Unset, str] = UNSET,
) -> dict[str, Any]:
    params: dict[str, Any] = {}

    params["revision"] = revision

    params = {
        k: v for k, v in params.items() if v is not UNSET and v is not None
    }

    _kwargs: dict[str, Any] = {
        "method": "get",
        "url": f"/projects/{project_id}/spaces/{space_id}/pages/{page_name}/attachments/{attachment_id}/content",
        "params": params,
    }

    return _kwargs


def _parse_response(
    *, client: Union[AuthenticatedClient, Client], response: httpx.Response
) -> Union[Any, Errors] | None:
    if response.status_code == 200:
        response_200 = cast(Any, None)
        return response_200
    if response.status_code == 400:
        response_400 = Errors.from_dict(response.json())

        return response_400
    if response.status_code == 401:
        response_401 = Errors.from_dict(response.json())

        return response_401
    if response.status_code == 403:
        response_403 = Errors.from_dict(response.json())

        return response_403
    if response.status_code == 404:
        response_404 = Errors.from_dict(response.json())

        return response_404
    if response.status_code == 406:
        response_406 = Errors.from_dict(response.json())

        return response_406
    if response.status_code == 500:
        response_500 = Errors.from_dict(response.json())

        return response_500
    if response.status_code == 503:
        response_503 = Errors.from_dict(response.json())

        return response_503
    if client.raise_on_unexpected_status:
        raise errors.UnexpectedStatus(response.status_code, response.content)
    return None


def _build_response(
    *, client: Union[AuthenticatedClient, Client], response: httpx.Response
) -> Response[Union[Any, Errors]]:
    return Response(
        status_code=HTTPStatus(response.status_code),
        content=response.content,
        headers=response.headers,
        parsed=_parse_response(client=client, response=response),
    )


def sync_detailed(
    project_id: str,
    space_id: str,
    page_name: str,
    attachment_id: str,
    *,
    client: Union[AuthenticatedClient, Client],
    revision: Union[Unset, str] = UNSET,
) -> Response[Union[Any, Errors]]:
    """Downloads the file content for a specified Page Attachment.

    Args:
        project_id (str):
        space_id (str):
        page_name (str):
        attachment_id (str):
        revision (Union[Unset, str]):

    Raises:
        errors.UnexpectedStatus: If the server returns an undocumented status code and Client.raise_on_unexpected_status is True.
        httpx.TimeoutException: If the request takes longer than Client.timeout.

    Returns:
        Response[Union[Any, Errors]]
    """

    kwargs = _get_kwargs(
        project_id=project_id,
        space_id=space_id,
        page_name=page_name,
        attachment_id=attachment_id,
        revision=revision,
    )

    response = client.get_httpx_client().request(
        **kwargs,
    )

    return _build_response(client=client, response=response)


def sync(
    project_id: str,
    space_id: str,
    page_name: str,
    attachment_id: str,
    *,
    client: Union[AuthenticatedClient, Client],
    revision: Union[Unset, str] = UNSET,
) -> Union[Any, Errors] | None:
    """Downloads the file content for a specified Page Attachment.

    Args:
        project_id (str):
        space_id (str):
        page_name (str):
        attachment_id (str):
        revision (Union[Unset, str]):

    Raises:
        errors.UnexpectedStatus: If the server returns an undocumented status code and Client.raise_on_unexpected_status is True.
        httpx.TimeoutException: If the request takes longer than Client.timeout.

    Returns:
        Union[Any, Errors]
    """

    return sync_detailed(
        project_id=project_id,
        space_id=space_id,
        page_name=page_name,
        attachment_id=attachment_id,
        client=client,
        revision=revision,
    ).parsed


async def asyncio_detailed(
    project_id: str,
    space_id: str,
    page_name: str,
    attachment_id: str,
    *,
    client: Union[AuthenticatedClient, Client],
    revision: Union[Unset, str] = UNSET,
) -> Response[Union[Any, Errors]]:
    """Downloads the file content for a specified Page Attachment.

    Args:
        project_id (str):
        space_id (str):
        page_name (str):
        attachment_id (str):
        revision (Union[Unset, str]):

    Raises:
        errors.UnexpectedStatus: If the server returns an undocumented status code and Client.raise_on_unexpected_status is True.
        httpx.TimeoutException: If the request takes longer than Client.timeout.

    Returns:
        Response[Union[Any, Errors]]
    """

    kwargs = _get_kwargs(
        project_id=project_id,
        space_id=space_id,
        page_name=page_name,
        attachment_id=attachment_id,
        revision=revision,
    )

    response = await client.get_async_httpx_client().request(**kwargs)

    return _build_response(client=client, response=response)


async def asyncio(
    project_id: str,
    space_id: str,
    page_name: str,
    attachment_id: str,
    *,
    client: Union[AuthenticatedClient, Client],
    revision: Union[Unset, str] = UNSET,
) -> Union[Any, Errors] | None:
    """Downloads the file content for a specified Page Attachment.

    Args:
        project_id (str):
        space_id (str):
        page_name (str):
        attachment_id (str):
        revision (Union[Unset, str]):

    Raises:
        errors.UnexpectedStatus: If the server returns an undocumented status code and Client.raise_on_unexpected_status is True.
        httpx.TimeoutException: If the request takes longer than Client.timeout.

    Returns:
        Union[Any, Errors]
    """

    return (
        await asyncio_detailed(
            project_id=project_id,
            space_id=space_id,
            page_name=page_name,
            attachment_id=attachment_id,
            client=client,
            revision=revision,
        )
    ).parsed



# Copyright DB InfraGO AG and contributors
# SPDX-License-Identifier: Apache-2.0

from collections.abc import Mapping
from typing import (
    TYPE_CHECKING,
    Any,
    TypeVar,
    Union,
)

from attrs import define as _attrs_define
from attrs import field as _attrs_field

from ..types import UNSET, Unset

if TYPE_CHECKING:
    from ..models.testrecords_single_get_response_data_relationships_defect import (
        TestrecordsSingleGetResponseDataRelationshipsDefect,
    )
    from ..models.testrecords_single_get_response_data_relationships_executed_by import (
        TestrecordsSingleGetResponseDataRelationshipsExecutedBy,
    )
    from ..models.testrecords_single_get_response_data_relationships_test_case import (
        TestrecordsSingleGetResponseDataRelationshipsTestCase,
    )


T = TypeVar("T", bound="TestrecordsSingleGetResponseDataRelationships")


@_attrs_define
class TestrecordsSingleGetResponseDataRelationships:
    """
    Attributes:
        defect (Union[Unset, TestrecordsSingleGetResponseDataRelationshipsDefect]):
        executed_by (Union[Unset, TestrecordsSingleGetResponseDataRelationshipsExecutedBy]):
        test_case (Union[Unset, TestrecordsSingleGetResponseDataRelationshipsTestCase]):
    """

    defect: Union[
        Unset, "TestrecordsSingleGetResponseDataRelationshipsDefect"
    ] = UNSET
    executed_by: Union[
        Unset, "TestrecordsSingleGetResponseDataRelationshipsExecutedBy"
    ] = UNSET
    test_case: Union[
        Unset, "TestrecordsSingleGetResponseDataRelationshipsTestCase"
    ] = UNSET
    additional_properties: dict[str, Any] = _attrs_field(
        init=False, factory=dict
    )

    def to_dict(self) -> dict[str, Any]:
        defect: Union[Unset, dict[str, Any]] = UNSET
        if not isinstance(self.defect, Unset):
            defect = self.defect.to_dict()

        executed_by: Union[Unset, dict[str, Any]] = UNSET
        if not isinstance(self.executed_by, Unset):
            executed_by = self.executed_by.to_dict()

        test_case: Union[Unset, dict[str, Any]] = UNSET
        if not isinstance(self.test_case, Unset):
            test_case = self.test_case.to_dict()

        field_dict: dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update({})
        if defect is not UNSET:
            field_dict["defect"] = defect
        if executed_by is not UNSET:
            field_dict["executedBy"] = executed_by
        if test_case is not UNSET:
            field_dict["testCase"] = test_case

        return field_dict

    @classmethod
    def from_dict(cls: type[T], src_dict: Mapping[str, Any]) -> T:
        from ..models.testrecords_single_get_response_data_relationships_defect import (
            TestrecordsSingleGetResponseDataRelationshipsDefect,
        )
        from ..models.testrecords_single_get_response_data_relationships_executed_by import (
            TestrecordsSingleGetResponseDataRelationshipsExecutedBy,
        )
        from ..models.testrecords_single_get_response_data_relationships_test_case import (
            TestrecordsSingleGetResponseDataRelationshipsTestCase,
        )

        d = dict(src_dict)
        _defect = d.pop("defect", UNSET)
        defect: Union[
            Unset, TestrecordsSingleGetResponseDataRelationshipsDefect
        ]
        if isinstance(_defect, Unset):
            defect = UNSET
        else:
            defect = (
                TestrecordsSingleGetResponseDataRelationshipsDefect.from_dict(
                    _defect
                )
            )

        _executed_by = d.pop("executedBy", UNSET)
        executed_by: Union[
            Unset, TestrecordsSingleGetResponseDataRelationshipsExecutedBy
        ]
        if isinstance(_executed_by, Unset):
            executed_by = UNSET
        else:
            executed_by = TestrecordsSingleGetResponseDataRelationshipsExecutedBy.from_dict(
                _executed_by
            )

        _test_case = d.pop("testCase", UNSET)
        test_case: Union[
            Unset, TestrecordsSingleGetResponseDataRelationshipsTestCase
        ]
        if isinstance(_test_case, Unset):
            test_case = UNSET
        else:
            test_case = TestrecordsSingleGetResponseDataRelationshipsTestCase.from_dict(
                _test_case
            )

        testrecords_single_get_response_data_relationships_obj = cls(
            defect=defect,
            executed_by=executed_by,
            test_case=test_case,
        )

        testrecords_single_get_response_data_relationships_obj.additional_properties = d
        return testrecords_single_get_response_data_relationships_obj

    @property
    def additional_keys(self) -> list[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



# Copyright DB InfraGO AG and contributors
# SPDX-License-Identifier: Apache-2.0

import datetime
from collections.abc import Mapping
from typing import (
    TYPE_CHECKING,
    Any,
    TypeVar,
    Union,
)

from attrs import define as _attrs_define
from attrs import field as _attrs_field
from dateutil.parser import isoparse

from ..types import UNSET, Unset

if TYPE_CHECKING:
    from ..models.collections_single_get_response_data_attributes_description import (
        CollectionsSingleGetResponseDataAttributesDescription,
    )


T = TypeVar("T", bound="CollectionsSingleGetResponseDataAttributes")


@_attrs_define
class CollectionsSingleGetResponseDataAttributes:
    """
    Attributes:
        closed_on (Union[Unset, datetime.datetime]):  Example: 1970-01-01T00:00:00Z.
        created (Union[Unset, datetime.datetime]):  Example: 1970-01-01T00:00:00Z.
        description (Union[Unset, CollectionsSingleGetResponseDataAttributesDescription]):
        id (Union[Unset, str]):  Example: ID.
        name (Union[Unset, str]):  Example: Name.
        updated (Union[Unset, datetime.datetime]):  Example: 1970-01-01T00:00:00Z.
    """

    closed_on: Union[Unset, datetime.datetime] = UNSET
    created: Union[Unset, datetime.datetime] = UNSET
    description: Union[
        Unset, "CollectionsSingleGetResponseDataAttributesDescription"
    ] = UNSET
    id: Union[Unset, str] = UNSET
    name: Union[Unset, str] = UNSET
    updated: Union[Unset, datetime.datetime] = UNSET
    additional_properties: dict[str, Any] = _attrs_field(
        init=False, factory=dict
    )

    def to_dict(self) -> dict[str, Any]:
        closed_on: Union[Unset, str] = UNSET
        if not isinstance(self.closed_on, Unset):
            closed_on = self.closed_on.isoformat()

        created: Union[Unset, str] = UNSET
        if not isinstance(self.created, Unset):
            created = self.created.isoformat()

        description: Union[Unset, dict[str, Any]] = UNSET
        if not isinstance(self.description, Unset):
            description = self.description.to_dict()

        id = self.id

        name = self.name

        updated: Union[Unset, str] = UNSET
        if not isinstance(self.updated, Unset):
            updated = self.updated.isoformat()

        field_dict: dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update({})
        if closed_on is not UNSET:
            field_dict["closedOn"] = closed_on
        if created is not UNSET:
            field_dict["created"] = created
        if description is not UNSET:
            field_dict["description"] = description
        if id is not UNSET:
            field_dict["id"] = id
        if name is not UNSET:
            field_dict["name"] = name
        if updated is not UNSET:
            field_dict["updated"] = updated

        return field_dict

    @classmethod
    def from_dict(cls: type[T], src_dict: Mapping[str, Any]) -> T:
        from ..models.collections_single_get_response_data_attributes_description import (
            CollectionsSingleGetResponseDataAttributesDescription,
        )

        d = dict(src_dict)
        _closed_on = d.pop("closedOn", UNSET)
        closed_on: Union[Unset, datetime.datetime]
        if isinstance(_closed_on, Unset):
            closed_on = UNSET
        else:
            closed_on = isoparse(_closed_on)

        _created = d.pop("created", UNSET)
        created: Union[Unset, datetime.datetime]
        if isinstance(_created, Unset):
            created = UNSET
        else:
            created = isoparse(_created)

        _description = d.pop("description", UNSET)
        description: Union[
            Unset, CollectionsSingleGetResponseDataAttributesDescription
        ]
        if isinstance(_description, Unset):
            description = UNSET
        else:
            description = CollectionsSingleGetResponseDataAttributesDescription.from_dict(
                _description
            )

        id = d.pop("id", UNSET)

        name = d.pop("name", UNSET)

        _updated = d.pop("updated", UNSET)
        updated: Union[Unset, datetime.datetime]
        if isinstance(_updated, Unset):
            updated = UNSET
        else:
            updated = isoparse(_updated)

        collections_single_get_response_data_attributes_obj = cls(
            closed_on=closed_on,
            created=created,
            description=description,
            id=id,
            name=name,
            updated=updated,
        )

        collections_single_get_response_data_attributes_obj.additional_properties = d
        return collections_single_get_response_data_attributes_obj

    @property
    def additional_keys(self) -> list[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



# Copyright DB InfraGO AG and contributors
# SPDX-License-Identifier: Apache-2.0

import json
from collections.abc import Mapping
from io import BytesIO
from typing import (
    TYPE_CHECKING,
    Any,
    TypeVar,
    Union,
)

from attrs import define as _attrs_define
from attrs import field as _attrs_field

from .. import types
from ..types import UNSET, File, Unset

if TYPE_CHECKING:
    from ..models.document_attachments_list_post_request import (
        DocumentAttachmentsListPostRequest,
    )


T = TypeVar("T", bound="PostDocumentAttachmentsRequestBody")


@_attrs_define
class PostDocumentAttachmentsRequestBody:
    """
    Attributes:
        files (Union[Unset, list[File]]):
        resource (Union[Unset, DocumentAttachmentsListPostRequest]):
    """

    files: Union[Unset, list[File]] = UNSET
    resource: Union[Unset, "DocumentAttachmentsListPostRequest"] = UNSET
    additional_properties: dict[str, Any] = _attrs_field(
        init=False, factory=dict
    )

    def to_dict(self) -> dict[str, Any]:
        files: Union[Unset, list[types.FileTypes]] = UNSET
        if not isinstance(self.files, Unset):
            files = []
            for files_item_data in self.files:
                files_item = files_item_data.to_tuple()

                files.append(files_item)

        resource: Union[Unset, dict[str, Any]] = UNSET
        if not isinstance(self.resource, Unset):
            resource = self.resource.to_dict()

        field_dict: dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update({})
        if files is not UNSET:
            field_dict["files"] = files
        if resource is not UNSET:
            field_dict["resource"] = resource

        return field_dict

    def to_multipart(self) -> types.RequestFiles:
        files: types.RequestFiles = []

        if not isinstance(self.files, Unset):
            for files_item_element in self.files:
                files.append(("files", files_item_element.to_tuple()))

        if not isinstance(self.resource, Unset):
            files.append(
                (
                    "resource",
                    (
                        None,
                        json.dumps(self.resource.to_dict()).encode(),
                        "text/plain",
                    ),
                )
            )

        for prop_name, prop in self.additional_properties.items():
            files.append((prop_name, (None, str(prop).encode(), "text/plain")))

        return files

    @classmethod
    def from_dict(cls: type[T], src_dict: Mapping[str, Any]) -> T:
        from ..models.document_attachments_list_post_request import (
            DocumentAttachmentsListPostRequest,
        )

        d = dict(src_dict)
        files = []
        _files = d.pop("files", UNSET)
        for files_item_data in _files or []:
            files_item = File(payload=BytesIO(files_item_data))

            files.append(files_item)

        _resource = d.pop("resource", UNSET)
        resource: Union[Unset, DocumentAttachmentsListPostRequest]
        if isinstance(_resource, Unset):
            resource = UNSET
        else:
            resource = DocumentAttachmentsListPostRequest.from_dict(_resource)

        post_document_attachments_request_body_obj = cls(
            files=files,
            resource=resource,
        )

        post_document_attachments_request_body_obj.additional_properties = d
        return post_document_attachments_request_body_obj

    @property
    def additional_keys(self) -> list[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties
